@@ -1,5 +1,8 @@
 use crate::config::Config;
-use std::path::Path;
+use std::{
+    path::{Path, PathBuf},
+    sync::Arc,
+};
 
 /// All types that are possible arguments to [`cmd!`] have to implement this trait.
 pub trait CmdArgument {
@@ -18,26 +21,26 @@ where
     }
 }
 
-/// Arguments of type [`&str`] are passed into the child process
+/// Arguments of type [`&str`] are passed to the child process
 /// as arguments.
 impl CmdArgument for &str {
     #[doc(hidden)]
     fn prepare_config(self, config: &mut Config) {
-        config.arguments.push(self.to_string());
+        config.arguments.push(self.into());
     }
 }
 
-/// Arguments of type [`String`] are passed into the child process
+/// Arguments of type [`String`] are passed to the child process
 /// as arguments.
 impl CmdArgument for String {
     #[doc(hidden)]
     fn prepare_config(self, config: &mut Config) {
-        config.arguments.push(self);
+        config.arguments.push(self.into());
     }
 }
 
 /// See the [`CmdArgument`] implementation for [`Split`] below.
-pub struct Split<'a>(pub &'a str);
+pub struct Split<T: AsRef<str>>(pub T);
 
 /// Splits the contained string by whitespace (using [`split_whitespace`])
 /// and uses the resulting words as separate arguments.
@@ -47,6 +50,9 @@ pub struct Split<'a>(pub &'a str);
 ///
 /// let StdoutTrimmed(output) = cmd!(Split("echo foo"));
 /// assert_eq!(output, "foo");
+///
+/// let StdoutTrimmed(output) = cmd!(Split(format!("echo {}", 100)));
+/// assert_eq!(output, "100");
 /// ```
 ///
 /// Since this is such a common case, `cradle` also provides a syntactic shortcut
@@ -60,11 +66,11 @@ pub struct Split<'a>(pub &'a str);
 /// ```
 ///
 /// [`split_whitespace`]: str::split_whitespace
-impl<'a> CmdArgument for Split<'a> {
+impl<T: AsRef<str>> CmdArgument for Split<T> {
     #[doc(hidden)]
     fn prepare_config(self, config: &mut Config) {
-        for argument in self.0.split_whitespace() {
-            config.arguments.push(argument.to_string());
+        for argument in self.0.as_ref().split_whitespace() {
+            argument.prepare_config(config);
         }
     }
 }
@@ -215,7 +221,7 @@ pub struct CurrentDir<T: AsRef<Path>>(pub T);
 /// ```
 /// use cradle::*;
 ///
-/// # #[cfg(target_os = "linux")]
+/// # #[cfg(linux)]
 /// # {
 /// let StdoutTrimmed(output) = cmd!("pwd", CurrentDir("/tmp"));
 /// assert_eq!(output, "/tmp");
@@ -232,3 +238,65 @@ where
         config.working_directory = Some(self.0.as_ref().to_owned());
     }
 }
+
+/// Arguments of type [`PathBuf`] are passed to the child process
+/// as arguments.
+///
+/// ```
+/// use cradle::*;
+/// use std::path::PathBuf;
+///
+/// let current_dir: PathBuf = std::env::current_dir().unwrap();
+/// cmd_unit!("ls", current_dir);
+/// ```
+impl CmdArgument for PathBuf {
+    #[doc(hidden)]
+    fn prepare_config(self, config: &mut Config) {
+        config.arguments.push(self.into());
+    }
+}
+
+/// Arguments of type [`&Path`] are passed to the child process
+/// as arguments.
+///
+/// ```
+/// use cradle::*;
+/// use std::path::Path;
+///
+/// let file: &Path = Path::new("./foo");
+/// cmd_unit!("touch", file);
+/// ```
+///
+/// [`&Path`]: std::path::Path
+impl CmdArgument for &Path {
+    #[doc(hidden)]
+    fn prepare_config(self, config: &mut Config) {
+        self.to_path_buf().prepare_config(config);
+    }
+}
+
+/// See the [`CmdArgument`] implementation for [`Stdin`] below.
+pub struct Stdin<T: Into<String>>(pub T);
+
+/// Writes the given [`&str`] to the child's standard input.
+/// If `Stdin` is used multiple times,
+/// all given strings will be written to the child's standard input in order.
+///
+/// ```
+/// use cradle::*;
+///
+/// # #[cfg(linux)]
+/// # {
+/// let StdoutUntrimmed(output) = cmd!("sort", Stdin("foo\nbar\n"));
+/// assert_eq!(output, "bar\nfoo\n");
+/// # }
+/// ```
+impl<T> CmdArgument for Stdin<T>
+where
+    T: Into<String>,
+{
+    #[doc(hidden)]
+    fn prepare_config(self, config: &mut Config) {
+        Arc::make_mut(&mut config.stdin).push(self.0.into());
+    }
+}
